// license: public domain
// authors: jonas.sundstrom@kirilla.com

//
// TODO: proper locking      <<---------
// 

#define DEBUG 0
#include <Debug.h>

#include <VolumeRoster.h>
#include <FindDirectory.h>
#include <Directory.h>
#include <Path.h>
#include <File.h>

#include "GenericSettings.h"

status_t  find_and_create_directory (directory_which a_which, BVolume * a_volume, const char * a_relative_path, BPath * a_full_path);


GenericSettings::GenericSettings()
:	BMessage		 	(),
	m_volume			(),
	m_base_dir			(B_USER_SETTINGS_DIRECTORY),
	m_relative_path		(),
	m_settings_filename	()
{
	PRINT(("GenericSettings()\n"));
}

GenericSettings::GenericSettings(BMessage a_message)
:	BMessage			(a_message),
	m_volume			(),
	m_base_dir			(B_USER_SETTINGS_DIRECTORY),
	m_relative_path		(),
	m_settings_filename	()
{
	PRINT(("GenericSettings(a_message)\n"));
}

GenericSettings::~GenericSettings()	
{
	m_volume.Unset();
}

status_t
GenericSettings::SetTo (const char *		a_settings_filename, 
					 BVolume *			a_volume,
					 directory_which	a_base_dir,
					 const char *		a_relative_path)
{
	status_t	status	=	B_OK;

	// copy to members
	m_base_dir			=	a_base_dir;
	m_relative_path		=	a_relative_path;
	m_settings_filename	=	a_settings_filename;
	
	// sanity check
	if (a_volume == NULL)
	{
		BVolumeRoster  volume_roster;
		volume_roster.GetBootVolume(& m_volume);
	}
	else
		m_volume  =  *(a_volume);
	
	status = m_volume.InitCheck();
	if (status != B_OK)
		return status;
	
	// InitCheck
	return InitCheck();
}

status_t	
GenericSettings::InitCheck	(void)
{
	BFile  settings_file;

	return GetSettingsFile	(& settings_file, B_READ_ONLY | B_CREATE_FILE);
}

status_t	
GenericSettings::GetSettingsFile	(BFile * a_settings_file, uint32 a_openmode)
{
	status_t	status	=	B_OK;
	BPath		settings_path;
	
	status	=	find_and_create_directory(m_base_dir, & m_volume, m_relative_path.String(), & settings_path);
	if (status != B_OK)
		return status;
		
	status	=	settings_path.Append (m_settings_filename.String());
	if (status != B_OK)
		return status;
	
	status	=	a_settings_file->SetTo (settings_path.Path(), a_openmode);
	if (status != B_OK)
		return status;
		
	return B_OK;
}

status_t
GenericSettings::ReadSettings	(void)
{
	PRINT(("GenericSettings::ReadSettings()\n"));

	status_t	status	=	B_OK;
	BFile		settings_file;
	
	status 	=	GetSettingsFile	(& settings_file, B_READ_ONLY);
	if (status != B_OK)
		return status;
	
	status	=	Unflatten(& settings_file);
	if (status != B_OK)
		return status;

	return B_OK;
}

status_t
GenericSettings::WriteSettings	(void)
{
	PRINT(("GenericSettings::WriteSettings()\n"));

	status_t	status	=	B_OK;
	BFile		settings_file;
	
	status 	=	GetSettingsFile	(& settings_file, B_WRITE_ONLY | B_ERASE_FILE);
	if (status != B_OK)
		return status;
	
	status	=	Flatten(& settings_file);
	if (status != B_OK)
		return status;

	return B_OK;
}

status_t  
find_and_create_directory (directory_which a_which, BVolume * a_volume, const char * a_relative_path, BPath * a_full_path)
{
	status_t	status	=	B_OK;
	BPath		path;
	mode_t		mask	=	0;
	
	status	=	find_directory (a_which, & path, true, a_volume); // create = true

	if (status != B_OK)
		return status;
	
	if (a_relative_path != NULL)
	{
		path.Append(a_relative_path);
		
		mask =	umask (0);
		umask (mask);
	
		status	=	create_directory (path.Path(), mask);
		if (status != B_OK)
			return status;
	}
	
	if (a_full_path != NULL)
	{
		status	=	a_full_path->SetTo(path.Path());
		if (status != B_OK)
			return status;
	}
	
	return B_OK;
}
