#include <stdlib.h>		
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include "scsi.h"

#include <Alert.h>
#include <Deskbar.h>
#include <Roster.h>
#include <NodeMonitor.h>

#define DEBUG 1
#include <Debug.h>

#include "MiniCDDefines.h"
#include "MiniCDReplicant.h"
#include "MiniCDLooper.h"

MiniCDLooper::MiniCDLooper	(DeskbarReplicant * replicant)
	:	BLooper			(MINICD_LOOPER),
		device_fd		(-1),
		m_media_state	(e_no_media),
		m_current_track	(0),
		m_num_of_tracks	(0),
		m_autoplay		(false),
		m_worker_thread	(NULL)
{
	m_replicant_view	=	replicant;
	
	LookForAlreadyMountedCD();
		
	m_volume_roster  =  new BVolumeRoster ();
	m_volume_roster->StartWatching(BMessenger(NULL, this));
}

MiniCDLooper::~MiniCDLooper (void)
{
	m_volume_roster->StopWatching();
	delete m_volume_roster;
	
	close (device_fd);
}

bool
MiniCDLooper::QuitRequested	(void)
{
	if (m_worker_thread != NULL)
		m_worker_thread->Kill();

	BDeskbar deskbar;
	deskbar.RemoveItem(MINICD_REPLICANT_VIEW_NAME); 

	return true;
}

void
MiniCDLooper::MessageReceived	(BMessage * message)
{
	switch(message->what) 
	{
		case B_ABOUT_REQUESTED:
				AboutRequested();
				break;	

		case B_NODE_MONITOR:
				HandleDeviceEvent(message);
				break;

		case 'xpnd':
				TellViewToExpand();
				break;
				
		case 'play':
				Play();
				break;
				
		case 'paus':
				Pause();
				break;
		
		case 'resu':
				PlayContinue();
				break;
		
		case 'stop':
				Stop();
				break;
		
		case 'next':
				SkipOneForward();
				break;
		
		case 'prev':
				SkipOneBackward();
				break;
			
		case 'ejct':
				Eject();
				break;
				
		case 'trak':
				{
					int32	track;
					message->FindInt32("track", & track);
					Play (track);
				}
				break;
		
		case 'sett':
				{
					bool autoplay;
					if (message->FindBool("autoplay", & autoplay) == B_OK)
						m_autoplay  =  autoplay;	
				}
				break;
		default: 
			BLooper::MessageReceived(message);
			break;
	}
}

void 
MiniCDLooper::OpenWebsite	(char * a_url)
{
	BMessage msg(B_ARGV_RECEIVED); 
	msg.AddString("argv", "app"); 
	msg.AddString("argv", a_url);  
	msg.AddInt32("argc", 2); 
	be_roster->Launch ("application/x-vnd.Be.URL.http", &msg ); 	
}

void
MiniCDLooper::AboutRequested	(void) 
{
	BAlert * alert = new BAlert("MiniCD", 
			"MiniCD " MINICD_APP_VERSION "\n\n"
			"Created by Jonas Sundström of Kirilla\n\n"
			"Public domain.",
			"Kirilla", "Close", NULL, B_WIDTH_AS_USUAL, B_IDEA_ALERT);

	int32 reply = alert->Go();
	
	if (reply == 0)
		OpenWebsite	("http://www.kirilla.com/");
}

void
MiniCDLooper::HandleDeviceEvent (BMessage * message) 
{
	PRINT(("MiniCDLooper::HandleDeviceEvent()\n"));

	status_t	status				=	B_OK;
	int32		opcode 				=	0;
	dev_t		mounted_device		=	0;	// mounted device
	dev_t		unmounted_device	=	0;	// unmounted device
	ino_t		mountdir			=	0;
	fs_info		fsinfo;
	
	message->FindInt32("opcode", & opcode);

	if (opcode == B_DEVICE_MOUNTED)
	{
		status = message->FindInt32("new device", & mounted_device);
		if (status != B_OK)
		{
			fprintf(stderr, "MiniCDLooper::HandleDeviceEvent()::FindInt32(new device):: %s\n", strerror(status));
			return;
		}
		
		status = message->FindInt64("directory", & mountdir);
		if (status != B_OK)
		{
			fprintf(stderr, "MiniCDLooper::HandleDeviceEvent()::FindInt32(directory):: %s\n", strerror(status));
			return;
		}
			
		status = fs_stat_dev (mounted_device, & fsinfo);
		if (status != B_OK)
		{
			fprintf(stderr, "MiniCDLooper::HandleDeviceEvent()::fs_stat_dev() %s\n", strerror(status));
			return;
		}
		
		bool new_cdda_device = false;
		
		new_cdda_device = IfCddaAddDeviceToList (& fsinfo);
		
		TellViewToExpand();
		
		if (m_autoplay)
			if (new_cdda_device)
				Play();
		
		printf("fsinfo_list.CountItems(): %ld\n", fsinfo_list.CountItems());
		
		// warning.. will autoplay current cd if any fs is mounted
		// 
		// create and use  IfCddaFs()
	}
	else if (opcode == B_DEVICE_UNMOUNTED)
	{
		status = message->FindInt32("device", & unmounted_device);
		if (status != B_OK)
		{
			fprintf(stderr, "MiniCDLooper::HandleDeviceEvent()::FindInt32(device):: %s\n", strerror(status));
			return;
		}
		
		int32  listindex  =  0;
		fs_info * fsinfo_listitem;

		while ((fsinfo_listitem = static_cast<fs_info *>(fsinfo_list.ItemAt(listindex))) != NULL)
		{
			if (fsinfo_listitem->dev == unmounted_device)
			{
				fsinfo_list.RemoveItem(listindex);
				
				DoThisIfNoDevice();
				
				CheckStatus();
				
				return;
			}
			++listindex;
		}
	}
}

void
MiniCDLooper::LookForAlreadyMountedCD (void)
{
	dev_t		device_num	=	0;
	int32		cookie		=	0;
	fs_info		info;

	while (1)
	{
		device_num =	next_dev(& cookie);
		
		if (device_num != B_BAD_VALUE)
		{	
			if(fs_stat_dev(device_num, &info) == B_OK)
			{
				bool success  =  IfCddaAddDeviceToList (& info);
				
				if (success)
					PostMessage('xpnd'); 

				// 'xpnd' is essentially the same as TellViewToExpand()
				// but postponed, to give Deskbar some time to start
				// before we hack it to re-arrange it's replicants
			}
		}
		else
			break;
	}
}

bool
MiniCDLooper::IfCddaAddDeviceToList (fs_info * fsinfo)
{
	if (! strcmp("cdda", fsinfo->fsh_name))
	{
		// add volume to list
		fs_info * new_fsinfo = new fs_info (*fsinfo);
		
		if (fsinfo_list.HasItem(new_fsinfo))
			return false;
		
		fsinfo_list.AddItem(new_fsinfo);
		
		// open device file descriptor
		InterfaceWithDevice (fsinfo);
		
		// start scsi state poller thread
		if (m_worker_thread == NULL)
		{
			m_worker_thread  =  new MiniCDWorker (this);
			m_worker_thread->Start();
		}
		
		return true;
	}
	else
		return false;
}

void
MiniCDLooper::TellViewToExpand (void)
{
	if (fsinfo_list.CountItems() > 0)
		if (m_replicant_view-> Window()-> Lock())
		{
			m_replicant_view-> ExpandOrContract(true);
			m_replicant_view-> Window()-> Unlock();
		}
}

void
MiniCDLooper::TellViewToContract (void)
{
	if (m_replicant_view-> Window()-> Lock())
	{
		m_replicant_view-> ExpandOrContract(false);
		m_replicant_view-> Window()-> Unlock();
	}
}

void
MiniCDLooper::DoThisIfNoDevice (void)
{
	if (fsinfo_list.CountItems() == 0)
	{
		TellViewToContract();
		
		status_t  status  =  B_OK;
		m_worker_thread->Quit();
		m_worker_thread->WaitForThread(& status);
		m_worker_thread  =  NULL;
	}
}

status_t
MiniCDLooper::InterfaceWithDevice (fs_info * fsinfo)
{
	if (device_fd != -1)
		close (device_fd);

	device_fd  =  open (fsinfo->device_name, O_RDONLY); 

	if (device_fd < 0) 
		return errno;

	return B_OK;
}	

status_t
MiniCDLooper::GetCurrentTrack (int32 * current_track)
{
	scsi_position	pos;
	status_t  		status	=	B_DEV_NO_MEDIA;

	ioctl(device_fd, B_GET_MEDIA_STATUS, & status, sizeof (status));

	if (status != B_OK)
		return status;

	status = ioctl(device_fd, B_SCSI_GET_POSITION, & pos);
	if (status != B_OK)
		return status;
	
	if (! pos.position[1] || pos.position[1] >= 0x13 || (pos.position[1] == 0x12 && ! pos.position[6]))
	{
//		current_track = 0;
		*current_track = m_current_track;
		return B_OK;
	}
	else
	{
		*current_track = pos.position[6];
		return B_OK;
	}
}

status_t 
MiniCDLooper::GetNumOfTracks (int32 * num_of_tracks) 
{
	scsi_toc toc;
	status_t status = ioctl (device_fd, B_SCSI_GET_TOC, & toc);

	if (status != B_OK)
	{
		PRINT(("error %s pausing\n", strerror(status)));
		return status;
	}
	
	*num_of_tracks  =  toc.toc_data[3];
	return B_OK;
}

void 
MiniCDLooper::Pause (void)
{
	status_t status = ioctl(device_fd, B_SCSI_PAUSE_AUDIO);
	if (status != B_OK) 
		PRINT(("error %s pausing\n", strerror(status)));
}

void 
MiniCDLooper::Play (int32 track_num = 0)
{
	// avoid restarting the current track
	{
		int32  current_track  =  0;
		GetCurrentTrack (& current_track);

		if (m_media_state == e_media_playing
			&& track_num == current_track)
			return;
	}

	if (m_media_state == e_media_paused)
		if (track_num == 0)
		{
			PlayContinue();
			return;
		}
	
	scsi_play_track  scsi_track;

	scsi_track.start_track = m_current_track;
	scsi_track.start_index = 1;
	scsi_track.end_track = 99;
	scsi_track.end_index = 1;

	if (track_num > 0)
		scsi_track.start_track  =  track_num;
	
	status_t status = ioctl(device_fd, B_SCSI_PLAY_TRACK, & scsi_track);
	if (status != B_OK)
		PRINT(("error %s playing track\n", strerror(status)));
}

void
MiniCDLooper::PlayContinue (void)
{
	status_t  status  =  ioctl (device_fd, B_SCSI_RESUME_AUDIO);

	if (status != B_OK) 
		PRINT(("error %s resuming\n", strerror(status)));
}

void 
MiniCDLooper::Stop (void)
{
	if (m_media_state == e_media_stopped)
	{
		Play(1);
		Pause();
	}

	status_t  status  =  ioctl (device_fd, B_SCSI_STOP_AUDIO);

	if (status != B_OK)
		PRINT(("error %s stopping\n", strerror(status)));
}

void 
MiniCDLooper::Eject (void)
{
	status_t status = ioctl(device_fd, B_EJECT_DEVICE);

	if (status == B_OK)
	{
		CheckStatus();
	}
	else
	{
		PRINT(("error %s ejecting\n", strerror(status)));
	}
}

void 
MiniCDLooper::SkipOneForward (void)
{
	int32 track;
	int32 num_of_tracks;

	GetCurrentTrack (& track);
	GetNumOfTracks (& num_of_tracks);

	if (track < num_of_tracks)	// avoid skipping past the last track
		track += 1;
	
	if (! (track == m_current_track))	// avoid restarting the last track
		Play (track);
}

void 
MiniCDLooper::SkipOneBackward (void)
{
	if (m_current_track > 1)
		Play (m_current_track - 1);
}

void
MiniCDLooper::CheckStatus (void)
{
	scsi_position	position;
	status_t		status			=	B_DEV_NO_MEDIA;
	int32			media_state		=	0;
	int32			current_track	=	0;
	int32			num_of_tracks	=	0;
	
	ioctl (device_fd, B_GET_MEDIA_STATUS, & status, sizeof(status));
	
	if (status != B_OK)
	{
		media_state  =  e_no_media;	// no disc
		
		if (m_replicant_view-> Window()-> Lock())
		{
			m_replicant_view-> SetState (e_no_media);
			m_replicant_view-> Window()-> Unlock();
		}	
	}
	else
	{
		status	=	ioctl(device_fd, B_SCSI_GET_POSITION, & position);
		
		current_track	=	position.position[6];
	
		if (status != B_OK)
		{
			media_state  =  e_no_media;	// no disc
			
		}
		else if ((!position.position[1]) || (position.position[1] >= 0x13) || ((position.position[1] == 0x12) && (!position.position[6])))
		{
			media_state  =  e_media_stopped;	// stopped
			GetNumOfTracks(& num_of_tracks);
		}
		else if (position.position[1] == 0x11)
		{
			media_state  =  e_media_playing;	// playing
			GetCurrentTrack(& current_track);
			GetNumOfTracks(& num_of_tracks);
		}
		else									
		{
			media_state  =  e_media_paused;	// paused
			GetCurrentTrack(& current_track);
			GetNumOfTracks(& num_of_tracks);
		}
	}

	if (media_state != m_media_state || current_track != m_current_track || num_of_tracks != m_num_of_tracks)
	{
		// update members
		m_media_state  =  media_state;
		m_current_track  =  current_track;
		m_num_of_tracks  =  num_of_tracks;

		// update view
		if (m_replicant_view-> Window()-> Lock())
		{
			m_replicant_view-> SetState (m_media_state);
			m_replicant_view-> SetCurrentTrack (m_current_track);
			m_replicant_view-> SetNumOfTracks (m_num_of_tracks);
			m_replicant_view-> Invalidate();
			m_replicant_view-> Window()-> Unlock();
		}	
	}
}
