#define DEBUG 1
#include <Debug.h>

#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include <Alert.h>
#include <Beep.h>
#include <Bitmap.h>
#include <DataIO.h>
#include <Deskbar.h>
#include <Directory.h>
#include <Entry.h>
#include <File.h>
#include <FindDirectory.h>
#include <Menu.h>
#include <MenuItem.h>
#include <Path.h>
#include <PopUpMenu.h>
#include <Resources.h>
#include <Roster.h>
#include <String.h>
#include <TranslationUtils.h>
#include <VolumeRoster.h>

#include "MiniCDDefines.h"
#include "MiniCDReplicant.h"
#include "MiniCDLooper.h"

const rgb_color COLOR_FOREGROUND	=	{ 0, 0, 0 };
const rgb_color COLOR_BACKGROUND	=	{ 255, 203, 102 };
const rgb_color COLOR_WHITE			=	{ 255, 255, 255 };
const rgb_color COLOR_BLACK			=	{ 0, 0, 0 };
const rgb_color COLOR_GREY			=	{ 150, 150, 150 };

#define  SMALL_ICON_RECT  BRect(0,0,15,15)

//enum	{ up, down, left, right	};

DeskbarReplicant::DeskbarReplicant(void)
	: BView(BRect(0, 0, 1, 1), 0, 0, 0)		
{
	m_looper_msgr	=	NULL;	// for the destructor
}

DeskbarReplicant::~DeskbarReplicant(void)	
{
	// Assure that the looper is dead and gone
	if (m_looper_msgr != NULL)
		if (m_looper_msgr->IsValid())
		{
			status_t  status  =  B_OK;
			thread_id  looper  =  m_looper->Thread();
			kill_thread(looper);	
			wait_for_thread(looper, & status);
		}
		
	// Deskbar unloads a replicant if its executable file looses it's
	// "be:deskbar_item_status" attribute, for example when recompiling.
	//  
	// Quitting, as implemented in our specific replicant,
	// is bypassed, which is why we have to take this extra step 
	// in the destructor of the replicant.
	//			
	// If we don't assure that the looper is gone
	// the looper will be left, running, in Deskbar space
	// and loading an updated (read: re-compiled) replicant
	// will simply revive the old one.. (since it was not
	// successfully unloaded)
}

DeskbarReplicant::DeskbarReplicant(BMessage * message)
:	BView			(BRect(0,0,15,15), MINICD_REPLICANT_VIEW_NAME, B_FOLLOW_NONE, B_WILL_DRAW),
	m_settings		(BMessage('sett')),
	m_looper		(NULL),
	m_looper_msgr	(NULL),
	m_popup_menu	(NULL),
	m_cd_icon		(NULL),
	m_play_icon		(NULL),
	m_pause_icon	(NULL),
	m_stop_icon		(NULL),
	m_previous_icon	(NULL),
	m_next_icon		(NULL),	
	m_is_expanded	(false),
	m_media_state	(e_no_media),
	m_current_track	(0),
	m_num_of_tracks	(0),
	m_autoplay		(false)
{
	LoadSettings();

	LoadIcons();

	SetFont(be_plain_font);
}

__declspec(dllexport) DeskbarReplicant * 
DeskbarReplicant::Instantiate(BMessage* archive)
{
	if (! validate_instantiation(archive, MINICD_REPLICANT_CLASS)) 
		return 0;

	return new DeskbarReplicant(archive);
}

status_t 
DeskbarReplicant::Archive(BMessage * archive, bool deep) const
{
	// Tell the overridden BView to archive itself.
	BView::Archive(archive, deep);
  
	// Store our signature and the name of our class in the archive.
	archive->AddString("add_on", MINICD_APP_SIGNATURE);
	archive->AddString("class", MINICD_REPLICANT_CLASS);

	return B_OK;
}

void
DeskbarReplicant::AttachedToWindow	(void)
{
	SetViewColor(Parent()->ViewColor());

	// start looper thread
	m_looper	=	new MiniCDLooper	(this);
	m_looper->Run();
	
	// set up communications
	m_looper_msgr	=	new BMessenger(NULL, m_looper);
	
	// propagate settings to looper
	m_looper_msgr->SendMessage(& m_settings);
	
	CreatePopup();
}

void 
DeskbarReplicant::Draw(BRect updateRect)
{
	// SetLowColor(COLOR_BACKGROUND);
	// SetHighColor(COLOR_FOREGROUND);
	SetDrawingMode(B_OP_ALPHA);
	
	if (! m_is_expanded)
	{
		MovePenTo(0,0);
		DrawBitmap(m_cd_icon);
	}
	else
	{
		MovePenTo(0,0);
		if (m_media_state == e_media_playing)
			DrawBitmap(m_pause_icon);
		else
			DrawBitmap(m_play_icon);
			
		MovePenTo(16,0);
		DrawBitmap(m_stop_icon);
		MovePenTo(32,0);
		DrawBitmap(m_previous_icon);
		MovePenTo(48,0);
		DrawBitmap(m_next_icon);
		
		if (!m_media_state == e_media_stopped || !m_media_state == e_no_media)
		{
			MovePenTo(70,13);
			char * string = new char [10];
			sprintf(string, "%ld / %ld", m_current_track, m_num_of_tracks);
			DrawString(string);
		}
	}
}


void
DeskbarReplicant::MouseDown(BPoint a_point) 
{
	uint32	button  =  0;
	BPoint	point (0,0);
	GetMouse (& point, & button);

	if ((m_media_state == e_no_media) || (button == B_SECONDARY_MOUSE_BUTTON))
	{
		CreatePopup();
		ConvertToScreen(& a_point);
		m_popup_menu->SetTargetForItems(this);
		m_popup_menu->Go(a_point, true, false, true);
		return;
	}
	
	if (PointIsWithinRect(a_point, BRect (0,0,15,15)))
		if (m_media_state == e_media_playing)
			m_looper_msgr->SendMessage('paus');
		else
			m_looper_msgr->SendMessage('play');
		
	if (PointIsWithinRect(a_point, BRect (16,0,31,15)))
		m_looper_msgr->SendMessage('stop');
		
	if (PointIsWithinRect(a_point, BRect (32,0,47,15)))
		m_looper_msgr->SendMessage('prev');

	if (PointIsWithinRect(a_point, BRect (48,0,63,15)))
		m_looper_msgr->SendMessage('next');
		
	if (PointIsWithinRect(a_point, BRect (64,0,80,15)))
		m_looper_msgr->SendMessage('quit');
}

bool
DeskbarReplicant::PointIsWithinRect (BPoint a_point, BRect a_rect)
{
	if (a_point.x < a_rect.left)	return false;
	if (a_point.x > a_rect.right)	return false;
	if (a_point.y < a_rect.top)		return false;
	if (a_point.y > a_rect.bottom)	return false;
	
	return true;
}

void
DeskbarReplicant::MessageReceived	(BMessage * a_message)
{
	switch(a_message->what) 
	{
		case 'auto':
			ToggleAutoplay();
			break;

		case 'play':	// fallthrough
		case 'paus':	// ... 
		case 'resu':	// .. 
		case 'stop':	// .
		case 'prev':
		case 'next':
		case 'ejct':
		case 'trak':
		case B_ABOUT_REQUESTED:
		case B_QUIT_REQUESTED:
		case MINICD_GOTO_WEBSITE:

			m_looper_msgr->SendMessage(a_message);
			break;

		case 'rset':
			m_media_state = e_no_media;
			m_current_track = 0;
			m_num_of_tracks = 0;
			break;
			
		default:
			BView::MessageReceived(a_message);
			break;
	}
}

void
DeskbarReplicant::CreatePopup	(void) 
{
	// remove existing
	if (m_popup_menu != NULL)
		delete m_popup_menu;

	// (re-)create
	m_popup_menu  =  new BPopUpMenu(MINICD_APP_NAME, false, false);

	BMenuItem * play		=	new BMenuItem("Play", new BMessage('play'));
	BMenuItem * pause		=	new BMenuItem("Pause", new BMessage('paus'));
	BMenuItem * stop		=	new BMenuItem("Stop", new BMessage('stop'));
	BMenuItem * next		=	new BMenuItem("Next", new BMessage('next'));
	BMenuItem * previous	=	new BMenuItem("Previous", new BMessage('prev'));
	BMenuItem * eject		=	new BMenuItem("Eject", new BMessage('ejct'));

	BMenuItem * autoplay	=	new BMenuItem("Autoplay", new BMessage('auto'));
	
	if (m_autoplay)
		autoplay->SetMarked (true);

	BMenuItem * about		=	new BMenuItem("About" B_UTF8_ELLIPSIS, new BMessage(B_ABOUT_REQUESTED));
	BMenuItem * quit		=	new BMenuItem("Quit", new BMessage(B_QUIT_REQUESTED));

	if (m_media_state == e_no_media)
	{
		m_popup_menu->AddItem(autoplay);
		m_popup_menu->AddSeparatorItem();
		m_popup_menu->AddItem(about);
		m_popup_menu->AddItem(quit);
	}
	
	if (m_media_state == e_media_playing)
	{
		m_popup_menu->AddItem(pause);
		m_popup_menu->AddItem(stop);
		m_popup_menu->AddItem(next);
		m_popup_menu->AddItem(previous);
		m_popup_menu->AddItem(eject);
		m_popup_menu->AddSeparatorItem();
		CreatePopupAddTracks();
		m_popup_menu->AddSeparatorItem();
		m_popup_menu->AddItem(autoplay);
		m_popup_menu->AddSeparatorItem();
		m_popup_menu->AddItem(about);
		m_popup_menu->AddItem(quit);
	}
	
	if (m_media_state == e_media_paused)
	{
		m_popup_menu->AddItem(play);
		m_popup_menu->AddItem(stop);
		m_popup_menu->AddItem(next);
		m_popup_menu->AddItem(previous);
		m_popup_menu->AddItem(eject);
		m_popup_menu->AddSeparatorItem();
		CreatePopupAddTracks();
		m_popup_menu->AddSeparatorItem();
		m_popup_menu->AddItem(autoplay);
		m_popup_menu->AddSeparatorItem();
		m_popup_menu->AddItem(about);
		m_popup_menu->AddItem(quit);
	}
	
	if (m_media_state == e_media_stopped)
	{
		m_popup_menu->AddItem(play);
		m_popup_menu->AddItem(next);
		m_popup_menu->AddItem(previous);
		m_popup_menu->AddItem(eject);
		m_popup_menu->AddSeparatorItem();
		CreatePopupAddTracks();
		m_popup_menu->AddSeparatorItem();
		m_popup_menu->AddItem(autoplay);
		m_popup_menu->AddSeparatorItem();
		m_popup_menu->AddItem(about);
		m_popup_menu->AddItem(quit);
	}

	status_t status = B_OK;
	status  =  m_popup_menu->SetTargetForItems(this);
}

void
DeskbarReplicant::CreatePopupAddTracks	(void) 
{
	BMenuItem * item;
	BMessage * message;
//	BString	string;
	char * string = new char [32];


	for (int32 index = 1; index <= m_num_of_tracks; index++)
	{
//		string.Truncate(0);
//		string += index;
		
		sprintf(string, "%ld", index);
		
		message = new BMessage ('trak');
		message->AddInt32("track", index);
		
		item = new BMenuItem(string, message);

		m_popup_menu->AddItem(item);
	}
}

void 
DeskbarReplicant::RearrangeDeskbar	(void)
{
	BDeskbar deskbar;
	
	if (deskbar.Location() == B_DESKBAR_TOP)
	{
		deskbar.SetLocation(B_DESKBAR_BOTTOM);
		deskbar.SetLocation(B_DESKBAR_TOP);
	}
	else if (deskbar.Location() == B_DESKBAR_BOTTOM)
	{
		deskbar.SetLocation(B_DESKBAR_TOP);
		deskbar.SetLocation(B_DESKBAR_BOTTOM);
	}
	else if (deskbar.IsExpanded())
	{
		deskbar.Expand(0);
		deskbar.Expand(1);
	}
	else
	{
		deskbar.Expand(1);
		deskbar.Expand(0);
	}	
}

void
DeskbarReplicant::ExpandOrContract (bool expand)
{
	if (expand && (! m_is_expanded))
	{
		ResizeTo (105, 15);
		m_is_expanded  =  true;
		RearrangeDeskbar();
	}
	else if ((! expand) && m_is_expanded)
	{
		ResizeTo (15, 15);
		m_is_expanded  =  false;
		RearrangeDeskbar();
	}
}

void
DeskbarReplicant::SetState (int32 a_media_state)
{
	m_media_state	=	a_media_state;
}

void
DeskbarReplicant::SetCurrentTrack (int32 a_current_track)
{
	m_current_track	=	a_current_track;
}

void
DeskbarReplicant::SetNumOfTracks (int32 a_num_of_tracks)
{
	m_num_of_tracks	=	a_num_of_tracks;
}

void
DeskbarReplicant::LoadIcons (void)
{
	status_t  status  =  B_OK;

	entry_ref	ref;
	BFile		file;
	BResources	resources;
	
	if (be_roster->FindApp(MINICD_APP_SIGNATURE, & ref) != B_OK)
		goto no_resources;
		
	status  =  file.SetTo(& ref, O_RDONLY);
	if (status != B_OK)
		goto no_resources;
		
	status  =  resources.SetTo(& file);
	if (status != B_OK)
		goto no_resources;
		
	m_cd_icon		=	GetBitmapFromResources (& resources, "cd.png");
	m_play_icon		=	GetBitmapFromResources (& resources, "play.png");
	m_pause_icon	=	GetBitmapFromResources (& resources, "pause.png");
	m_stop_icon		=	GetBitmapFromResources (& resources, "stop.png");
	m_next_icon		=	GetBitmapFromResources (& resources, "next.png");
	m_previous_icon	=	GetBitmapFromResources (& resources, "previous.png");
	
	no_resources: 
	
	if (m_cd_icon == NULL)
	{
		m_cd_icon = new BBitmap(SMALL_ICON_RECT, B_COLOR_8_BIT);
		m_cd_icon-> SetBits(& MiniCDIconBits, 256, 0, B_COLOR_8_BIT);
	}
	
	if (m_play_icon == NULL)
	{
		m_play_icon = new BBitmap(SMALL_ICON_RECT, B_COLOR_8_BIT);
		m_play_icon-> SetBits(& PlayIconBits, 256, 0, B_COLOR_8_BIT);
	}

	if (m_pause_icon == NULL)
	{
		m_pause_icon = new BBitmap(SMALL_ICON_RECT, B_COLOR_8_BIT);
		m_pause_icon-> SetBits(& PauseIconBits, 256, 0, B_COLOR_8_BIT);
	}
	
	if (m_stop_icon == NULL)
	{
		m_stop_icon = new BBitmap(SMALL_ICON_RECT, B_COLOR_8_BIT);
		m_stop_icon-> SetBits(& StopIconBits, 256, 0, B_COLOR_8_BIT);
	}
	
	if (m_next_icon == NULL)
	{
		m_next_icon = new BBitmap(SMALL_ICON_RECT, B_COLOR_8_BIT);
		m_next_icon-> SetBits(& NextIconBits, 256, 0, B_COLOR_8_BIT);
	}
	
	if (m_previous_icon == NULL)
	{
		m_previous_icon = new BBitmap(SMALL_ICON_RECT, B_COLOR_8_BIT);
		m_previous_icon-> SetBits(& PreviousIconBits, 256, 0, B_COLOR_8_BIT);
	}
}

BBitmap * 
DeskbarReplicant::GetBitmapFromResources (BResources * resources, const char * name)
{
	if (! resources->HasResource('PNG ', name))
		return NULL;
		
	size_t  size  =  0;
	const void *  data  =  NULL;

	data  =  resources->LoadResource('PNG ', name, & size);
	
	if (data == NULL)
		return NULL;
		
	BMemoryIO container (data, size);
		
	return BTranslationUtils::GetBitmap(& container);
}

void
DeskbarReplicant::LoadSettings (void)
{
	m_settings.SetTo(MINICD_SETTINGS_FILENAME, NULL, B_USER_SETTINGS_DIRECTORY, MINICD_SETTINGS_FOLDER);
	
	m_settings.ReadSettings();
	
	if (m_settings.FindBool("autoplay", & m_autoplay) != B_OK)
		m_autoplay  =  false;
}

void
DeskbarReplicant::ToggleAutoplay (void)
{
	// update member variable
	m_autoplay = (! m_autoplay);

	// save settings file
	m_settings.MakeEmpty();
	m_settings.AddBool("autoplay", m_autoplay);
	m_settings.WriteSettings();

	// tell looper
	m_settings.what = 'sett';
	m_looper_msgr->SendMessage(& m_settings);
}

