#ifndef _MINICDREPLICANT_H
#define _MINICDREPLICANT_H

#include <View.h>
#include <String.h>
#include <Directory.h>

#include "MiniCDLooper.h"
#include "MiniCDDefines.h"
#include "GenericSettings.h"

class DeskbarReplicant : public BView 
{
public:
	DeskbarReplicant(void);
	~DeskbarReplicant(void);

	DeskbarReplicant(BMessage * message);
	
	static DeskbarReplicant * Instantiate(BMessage * archive);

	virtual status_t Archive(BMessage* archive, bool deep = true) const;

	virtual void 	Draw				(BRect updateRect);
	virtual void 	MouseDown			(BPoint point);
	virtual void 	MessageReceived		(BMessage * message);
	virtual void 	AttachedToWindow	(void);
	
protected:
private:

friend class MiniCDLooper;

	void		LoadSettings			(void);
	void		LoadIcons				(void);
	BBitmap * 	GetBitmapFromResources	(BResources * resources, const char * name);
	
	void		CreatePopup				(void);
	void		CreatePopupAddTracks	(void);
	void		ToggleAutoplay 			(void);
	
	void		RearrangeDeskbar		(void);
	void		ExpandOrContract		(bool expand);	
	bool		PointIsWithinRect		(BPoint a_point, BRect a_rect);
	
	void		SetState		(int32 a_media_state);
	void		SetCurrentTrack	(int32 a_current_track);
	void		SetNumOfTracks	(int32 a_num_of_tracks);
	
	GenericSettings		m_settings;
	
	BLooper			*	m_looper;
	BMessenger		*	m_looper_msgr;
	BPopUpMenu		*	m_popup_menu;
	
	BBitmap			*	m_cd_icon;
	BBitmap			*	m_play_icon;
	BBitmap			*	m_pause_icon;
	BBitmap			*	m_stop_icon;
	BBitmap			*	m_previous_icon;
	BBitmap			*	m_next_icon;
		
	bool		m_is_expanded;
	int32		m_media_state;
	int32		m_current_track;
	int32		m_num_of_tracks;
	
	bool		m_autoplay;
	
};

// ********************************************************************************
const unsigned char MiniCDIconBits [] = {
	0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
	0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
	0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,
	0xff,0xff,0xff,0x00,0x00,0x1d,0x1e,0x1e,0x1e,0x1e,0x1d,0x00,0x00,0x0f,0xff,0xff,
	0xff,0xff,0x00,0x1d,0x1e,0x1a,0x1a,0x18,0x18,0x19,0x19,0x1e,0x1e,0x00,0x0f,0xff,
	0xff,0x00,0x60,0x41,0x1a,0x3f,0x14,0x1d,0x00,0x14,0x3f,0x19,0x41,0x60,0x00,0x0f,
	0xff,0x00,0x60,0x41,0x44,0xd8,0x1e,0x00,0x3f,0x1e,0xd8,0x44,0x41,0x60,0x00,0x0f,
	0xff,0x00,0x60,0x41,0x44,0x18,0x14,0x1e,0x00,0x14,0x1d,0x44,0x41,0x60,0x00,0x0f,
	0xff,0xff,0x00,0x1d,0x1e,0x3f,0x19,0x19,0x18,0x18,0x3f,0x1d,0x1d,0x00,0x0f,0x0f,
	0xff,0xff,0xff,0x00,0x00,0x1d,0x1e,0x1e,0x1e,0x1e,0x1d,0x00,0x00,0x0f,0x0f,0xff,
	0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x0f,0x0f,0x0f,0xff,0xff,
	0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x0f,0x0f,0x0f,0x0f,0x0f,0xff,0xff,0xff,0xff,
	0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
	0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
	0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
	0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff
};

const unsigned char PlayIconBits [] = {
	0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
	0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
	0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,
	0xff,0xff,0xff,0x00,0x00,0x1d,0x1e,0x1e,0x1e,0x1e,0x1d,0x00,0x00,0x0f,0xff,0xff,
	0xff,0xff,0x00,0x1d,0x1e,0x1a,0x1a,0x18,0x18,0x19,0x19,0x1e,0x1e,0x00,0x0f,0xff,
	0xff,0x00,0x00,0x41,0x1a,0x3f,0x14,0x1d,0x00,0x14,0x3f,0x19,0x41,0x60,0x00,0x0f,
	0xff,0x00,0x00,0x00,0x44,0xd8,0x1e,0x00,0x3f,0x1e,0xd8,0x44,0x41,0x60,0x00,0x0f,
	0xff,0x00,0x00,0x00,0x00,0x18,0x14,0x1e,0x00,0x14,0x1d,0x44,0x41,0x60,0x00,0x0f,
	0xff,0x00,0x00,0x00,0x00,0x00,0x19,0x19,0x18,0x18,0x3f,0x1d,0x1d,0x00,0x0f,0x0f,
	0xff,0x00,0x00,0x00,0x00,0x1b,0x1e,0x1e,0x1e,0x1e,0x1d,0x00,0x00,0x0f,0x0f,0xff,
	0xff,0x00,0x00,0x00,0x1b,0x00,0x00,0x00,0x00,0x00,0x00,0x0f,0x0f,0x0f,0xff,0xff,
	0xff,0x00,0x00,0xff,0xff,0xff,0xff,0x0f,0x0f,0x0f,0x0f,0x0f,0xff,0xff,0xff,0xff,
	0xff,0x1b,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
	0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
	0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
	0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff
};

const unsigned char PauseIconBits [] = {
	0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
	0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
	0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,
	0xff,0xff,0xff,0x00,0x00,0x1d,0x1e,0x1e,0x1e,0x1e,0x1d,0x00,0x00,0x0f,0xff,0xff,
	0xff,0xff,0x00,0x1d,0x1e,0x1a,0x1a,0x18,0x18,0x19,0x19,0x1e,0x1e,0x00,0x0f,0xff,
	0xff,0x00,0x60,0x41,0x1a,0x3f,0x14,0x1d,0x00,0x14,0x3f,0x19,0x41,0x60,0x00,0x0f,
	0xff,0x00,0x60,0x41,0x44,0xd8,0x1e,0x00,0x3f,0x1e,0xd8,0x44,0x41,0x60,0x00,0x0f,
	0xff,0x00,0x60,0x41,0x44,0x18,0x14,0x1e,0x00,0x14,0x1d,0x44,0x41,0x60,0x00,0x0f,
	0xff,0xff,0x00,0x00,0x1e,0x00,0x00,0x19,0x18,0x18,0x3f,0x1d,0x1d,0x00,0x0f,0x0f,
	0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x1e,0x1e,0x1e,0x1d,0x00,0x00,0x0f,0x0f,0xff,
	0xff,0xff,0x00,0x00,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x0f,0x0f,0x0f,0xff,0xff,
	0xff,0xff,0x00,0x00,0xff,0x00,0x00,0x0f,0x0f,0x0f,0x0f,0x0f,0xff,0xff,0xff,0xff,
	0xff,0xff,0x00,0x00,0xff,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
	0xff,0xff,0x00,0x00,0xff,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
	0xff,0xff,0x00,0x00,0xff,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
	0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff
};

const unsigned char StopIconBits [] = {
	0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
	0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
	0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,
	0xff,0xff,0xff,0x00,0x00,0x1d,0x1e,0x1e,0x1e,0x1e,0x1d,0x00,0x00,0x0f,0xff,0xff,
	0xff,0xff,0x00,0x1d,0x1e,0x1a,0x1a,0x18,0x18,0x19,0x19,0x1e,0x1e,0x00,0x0f,0xff,
	0xff,0x00,0x60,0x41,0x1a,0x3f,0x14,0x1d,0x00,0x14,0x3f,0x19,0x41,0x60,0x00,0x0f,
	0xff,0x00,0x60,0x41,0x44,0xd8,0x1e,0x00,0x3f,0x1e,0xd8,0x44,0x41,0x60,0x00,0x0f,
	0xff,0x00,0x60,0x41,0x44,0x18,0x14,0x1e,0x00,0x14,0x1d,0x44,0x41,0x60,0x00,0x0f,
	0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x19,0x18,0x18,0x3f,0x1d,0x1d,0x00,0x0f,0x0f,
	0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x1e,0x1e,0x1e,0x1d,0x00,0x00,0x0f,0x0f,0xff,
	0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0f,0x0f,0x0f,0xff,0xff,
	0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x0f,0x0f,0x0f,0x0f,0x0f,0xff,0xff,0xff,0xff,
	0xff,0xff,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
	0xff,0xff,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
	0xff,0xff,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
	0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff
};

const unsigned char NextIconBits [] = {
	0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
	0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
	0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,
	0xff,0xff,0xff,0x00,0x00,0x1d,0x1e,0x1e,0x1e,0x1e,0x1d,0x00,0x00,0x0f,0xff,0xff,
	0xff,0xff,0x00,0x1d,0x1e,0x1a,0x1a,0x18,0x18,0x19,0x19,0x1e,0x1e,0x00,0x0f,0xff,
	0xff,0x00,0x60,0x41,0x1a,0x3f,0x14,0x1d,0x00,0x14,0x3f,0x19,0x41,0x60,0x00,0x0f,
	0xff,0x00,0x60,0x41,0x44,0xd8,0x1e,0x00,0x3f,0x1e,0xd8,0x44,0x41,0x60,0x00,0x0f,
	0xff,0x00,0x60,0x41,0x44,0x18,0x14,0x1e,0x00,0x14,0x1d,0x44,0x41,0x60,0x00,0x0f,
	0xff,0xff,0x00,0x1d,0x1e,0x3f,0x19,0x19,0x18,0x18,0x3f,0x1d,0x1d,0x00,0x0f,0x0f,
	0xff,0xff,0xff,0x00,0x00,0x1d,0x00,0x1e,0x1e,0x00,0x1d,0x00,0x00,0x0f,0x0f,0xff,
	0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x0f,0x0f,0x0f,0xff,0xff,
	0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x0f,0x0f,0x00,0x00,0xff,0xff,0xff,0xff,
	0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0xff,0xff,0x00,0x00,0xff,0xff,0xff,
	0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0xff,0xff,0x00,0x00,0xff,0xff,0xff,0xff,
	0xff,0xff,0xff,0xff,0xff,0x00,0x00,0xff,0xff,0x00,0x00,0xff,0xff,0xff,0xff,0xff,
	0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff
};

const unsigned char PreviousIconBits [] = {
	0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
	0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
	0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,
	0xff,0xff,0xff,0x00,0x00,0x1d,0x1e,0x1e,0x1e,0x1e,0x1d,0x00,0x00,0x0f,0xff,0xff,
	0xff,0xff,0x00,0x1d,0x1e,0x1a,0x1a,0x18,0x18,0x19,0x19,0x1e,0x1e,0x00,0x0f,0xff,
	0xff,0x00,0x60,0x41,0x1a,0x3f,0x14,0x1d,0x00,0x14,0x3f,0x19,0x41,0x60,0x00,0x0f,
	0xff,0x00,0x60,0x41,0x44,0xd8,0x1e,0x00,0x3f,0x1e,0xd8,0x44,0x41,0x60,0x00,0x0f,
	0xff,0x00,0x60,0x41,0x44,0x18,0x14,0x1e,0x00,0x14,0x1d,0x44,0x41,0x60,0x00,0x0f,
	0xff,0xff,0x00,0x1d,0x1e,0x3f,0x19,0x19,0x18,0x18,0x3f,0x1d,0x1d,0x00,0x0f,0x0f,
	0xff,0xff,0xff,0x00,0x00,0x1d,0x1e,0x1e,0x00,0x1e,0x1d,0x00,0x00,0x0f,0x0f,0xff,
	0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0f,0x0f,0x0f,0xff,0xff,
	0xff,0xff,0x00,0x00,0xff,0xff,0x00,0x00,0x0f,0x0f,0x0f,0x0f,0xff,0xff,0xff,0xff,
	0xff,0x00,0x00,0xff,0xff,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
	0xff,0xff,0x00,0x00,0xff,0xff,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
	0xff,0xff,0xff,0x00,0x00,0xff,0xff,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
	0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff
};

#endif
